@@ -2,7 +2,7 @@
  *
  * raptor_general.c - Raptor general routines
  *
- * $Id: raptor_general.c,v 1.313 2005/02/01 22:00:36 cmdjb Exp $
+ * $Id: raptor_general.c,v 1.318 2005/05/19 09:42:18 cmdjb Exp $
  *
  * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -86,6 +86,9 @@ raptor_init(void)
   if(raptor_initialised)
     return;
 
+#ifdef RAPTOR_PARSER_GRDDL
+  raptor_init_parser_grddl();
+#endif
 #ifdef RAPTOR_PARSER_RSS
   raptor_init_parser_rss();
   raptor_init_serializer_rss10();
@@ -103,10 +106,15 @@ raptor_init(void)
   raptor_init_parser_rdfxml();
 #endif
 
+#ifdef RAPTOR_SERIALIZER_RDFXML
   raptor_init_serializer_rdfxml();
+#endif
+#ifdef RAPTOR_SERIALIZER_RDFXML_ABBREV
   raptor_init_serializer_rdfxmla();
-  
+#endif
+#ifdef RAPTOR_SERIALIZER_NTRIPLES
   raptor_init_serializer_ntriples();
+#endif
   /* raptor_init_serializer_simple(); */
 
   raptor_uri_init();
@@ -298,7 +306,7 @@ raptor_free_statement(raptor_statement *statement) {
     if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE)
       raptor_free_uri((raptor_uri*)statement->subject);
     else
-      RAPTOR_FREE(cstring, statement->subject);
+      RAPTOR_FREE(cstring, (void*)statement->subject);
   }
 
   if(statement->predicate) {
@@ -306,7 +314,7 @@ raptor_free_statement(raptor_statement *statement) {
        statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE)
       raptor_free_uri((raptor_uri*)statement->predicate);
     else
-      RAPTOR_FREE(cstring, statement->predicate);
+      RAPTOR_FREE(cstring, (void*)statement->predicate);
   }
 
   if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE) {
@@ -314,10 +322,10 @@ raptor_free_statement(raptor_statement *statement) {
       raptor_free_uri((raptor_uri*)statement->object);
   } else {
     if(statement->object)
-      RAPTOR_FREE(cstring, statement->object);
+      RAPTOR_FREE(cstring, (void*)statement->object);
 
     if(statement->object_literal_language)
-      RAPTOR_FREE(cstring, statement->object_literal_language);
+      RAPTOR_FREE(cstring, (void*)statement->object_literal_language);
     if(statement->object_literal_datatype)
       raptor_free_uri((raptor_uri*)statement->object_literal_datatype);
   }
@@ -587,6 +595,7 @@ raptor_statement_part_as_counted_string(const void *term,
       *s++ ='\0';
       break;
       
+    case RAPTOR_IDENTIFIER_TYPE_UNKNOWN:
     default:
       RAPTOR_FATAL2("Unknown type %d", type);
   }
@@ -675,6 +684,7 @@ raptor_print_statement_part_as_ntriples(FILE* stream,
       fputc('>', stream);
       break;
       
+    case RAPTOR_IDENTIFIER_TYPE_UNKNOWN:
     default:
       RAPTOR_FATAL2("Unknown type %d", type);
   }
@@ -798,6 +808,72 @@ raptor_calloc_memory(size_t nmemb, size_t size)
 }
 
 
+#if defined (RAPTOR_DEBUG) && defined(RAPTOR_MEMORY_SIGN)
+void*
+raptor_sign_malloc(size_t size)
+{
+  int *p;
+  
+  size += sizeof(int);
+  
+  p=(int*)malloc(size);
+  *p++ = RAPTOR_SIGN_KEY;
+  return p;
+}
+
+void*
+raptor_sign_calloc(size_t nmemb, size_t size)
+{
+  int *p;
+  
+  /* turn into bytes */
+  size = nmemb*size + sizeof(int);
+  
+  p=(int*)calloc(1, size);
+  *p++ = RAPTOR_SIGN_KEY;
+  return p;
+}
+
+void*
+raptor_sign_realloc(void *ptr, size_t size)
+{
+  int *p;
+
+  if(!ptr)
+    return raptor_sign_malloc(size);
+  
+  p=(int*)ptr;
+  p--;
+
+  if(*p != RAPTOR_SIGN_KEY)
+    RAPTOR_FATAL3("memory signature %08X != %08X", *p, RAPTOR_SIGN_KEY);
+
+  size += sizeof(int);
+  
+  p=(int*)realloc(p, size);
+  *p++= RAPTOR_SIGN_KEY;
+  return p;
+}
+
+void
+raptor_sign_free(void *ptr)
+{
+  int *p;
+
+  if(!ptr)
+    return;
+  
+  p=(int*)ptr;
+  p--;
+
+  if(*p != RAPTOR_SIGN_KEY)
+    RAPTOR_FATAL3("memory signature %08X != %08X", *p, RAPTOR_SIGN_KEY);
+
+  free(p);
+}
+#endif
+
+
 #if defined (RAPTOR_DEBUG) && defined(HAVE_DMALLOC_H) && defined(RAPTOR_MEMORY_DEBUG_DMALLOC)
 
 #undef malloc
