@@ -2,7 +2,7 @@
  *
  * raptor_internal.h - Redland Parser Toolkit for RDF (Raptor) internals
  *
- * $Id: raptor_internal.h,v 1.131 2005/02/01 22:00:36 cmdjb Exp $
+ * $Id: raptor_internal.h,v 1.135 2005/05/17 06:30:37 cmdjb Exp $
  *
  * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -46,15 +46,35 @@ extern "C" {
 #endif
 
 
+/* Can be over-ridden or undefined in a config.h file or -Ddefine */
+#ifndef RAPTOR_INLINE
+#define RAPTOR_INLINE inline
+#endif
+
 #ifdef LIBRDF_DEBUG
 #define RAPTOR_DEBUG 1
 #endif
 
+#if defined(RAPTOR_MEMORY_SIGN)
+#define RAPTOR_SIGN_KEY 0x08A61080
+void* raptor_sign_malloc(size_t size);
+void* raptor_sign_calloc(size_t nmemb, size_t size);
+void* raptor_sign_realloc(void *ptr, size_t size);
+void raptor_sign_free(void *ptr);
+  
+#define RAPTOR_MALLOC(type, size)   raptor_sign_malloc(size)
+#define RAPTOR_CALLOC(type, nmemb, size) raptor_sign_calloc(nmemb, size)
+#define RAPTOR_REALLOC(type, ptr, size) raptor_sign_realloc(ptr, size)
+#define RAPTOR_FREE(type, ptr)   raptor_sign_free(ptr)
+
+#else
 #define RAPTOR_MALLOC(type, size) malloc(size)
 #define RAPTOR_CALLOC(type, nmemb, size) calloc(nmemb, size)
 #define RAPTOR_REALLOC(type, ptr, size) realloc(ptr, size)
 #define RAPTOR_FREE(type, ptr)   free((void*)ptr)
 
+#endif
+
 #ifdef RAPTOR_DEBUG
 /* Debugging messages */
 #define RAPTOR_DEBUG1(msg) do {fprintf(stderr, "%s:%d:%s: " msg, __FILE__, __LINE__, __func__); } while(0)
@@ -189,6 +209,9 @@ typedef struct raptor_xml_entity_t raptor_xml_entity;
 
 /* raptor_libxml.c exports */
 extern void raptor_libxml_init(xmlSAXHandler *sax);
+extern void raptor_libxml_init_sax_error_handlers(xmlSAXHandler *sax);
+extern void raptor_libxml_init_generic_error_handlers(raptor_parser *rdf_parser);
+
 #ifdef RAPTOR_LIBXML_MY_ENTITIES
 extern void raptor_libxml_free_entities(raptor_parser *rdf_parser);
 #endif
@@ -632,6 +655,7 @@ void raptor_init_parser_rdfxml(void);
 void raptor_init_parser_ntriples(void);
 void raptor_init_parser_turtle(void);
 void raptor_init_parser_n3(void);
+void raptor_init_parser_grddl(void);
 
 /* raptor_rss.c */
 void raptor_init_parser_rss(void);
