@@ -2,7 +2,7 @@
  *
  * raptor_sax2.c - Raptor SAX2 API
  *
- * $Id: raptor_sax2.c,v 1.23 2005/01/15 17:19:43 cmdjb Exp $
+ * $Id: raptor_sax2.c,v 1.24 2005/05/17 22:44:52 cmdjb Exp $
  *
  * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -379,7 +379,7 @@ raptor_free_xml_element(raptor_xml_element *element)
     raptor_free_uri(element->base_uri);
 
   if(element->xml_language)
-    RAPTOR_FREE(cstring, element->xml_language);
+    RAPTOR_FREE(cstring, (void*)element->xml_language);
 
   raptor_free_qname(element->name);
 
