@@ -2,7 +2,7 @@
  *
  * raptor_serialize_xmlrdf.c - RDF/XML abbreviations serializer
  *
- * $Id: raptor_serialize_rdfxmla.c,v 1.4 2005/02/01 23:34:39 cmdjb Exp $
+ * $Id: raptor_serialize_rdfxmla.c,v 1.10 2005/06/07 22:48:41 cmdjb Exp $
  *
  * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
  * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
@@ -112,6 +112,9 @@ typedef struct {
   raptor_sequence *blanks;              /* blank subject items */
   raptor_sequence *nodes;               /* nodes */
   raptor_node *rdf_type;                /* rdf:type uri */
+
+  /* URI of rdf:XMLLiteral */
+  raptor_uri* rdf_xml_literal_uri;
 } raptor_rdfxmla_context;
 
 
@@ -243,20 +246,18 @@ raptor_new_qname_from_resource(raptor_serializer* serializer,
 
   uri_string = raptor_uri_as_counted_string(node->value.resource.uri, &uri_len);
 
-  p= uri_string + uri_len - name_len;
-  while(p >= uri_string) {
-    if(raptor_xml_name_check(p, name_len, 10))
+  p= uri_string;
+  name_len=uri_len;
+  while(name_len >0) {
+    if(raptor_xml_name_check(p, name_len, 10)) {
       name=p;
-    else if(name && p>uri_string &&
-            !raptor_xml_name_check(p-1, name_len+1, 10))
-      /* if next char would make name invalid, stop */
       break;
-    p--; name_len++;
+    }
+    p++; name_len--;
   }
       
-  if(!name) {
+  if(!name || (name == uri_string))
     return NULL;
-  }
 
   c=*name; *name='\0';
   ns_uri=raptor_new_uri(uri_string);
@@ -466,6 +467,7 @@ raptor_rdfxmla_emit_subject_list_items(raptor_serializer* serializer,
   raptor_rdfxmla_context* context=(raptor_rdfxmla_context*)serializer->context;
   int rv = 0;
   int i=0;
+  raptor_uri* base_uri=NULL;
   
   while (!rv && i < raptor_sequence_size(subject->list_items)) {
 
@@ -481,8 +483,9 @@ raptor_rdfxmla_emit_subject_list_items(raptor_serializer* serializer,
                                                        (unsigned char *)"li",
                                                        NULL);
     
-    element = raptor_new_xml_element(qname, NULL,
-                                     raptor_uri_copy(serializer->base_uri));
+    if(serializer->base_uri)
+      base_uri=raptor_uri_copy(serializer->base_uri);
+    element = raptor_new_xml_element(qname, NULL, base_uri);
 
     switch (object->type) {
       
@@ -502,6 +505,11 @@ raptor_rdfxmla_emit_subject_list_items(raptor_serializer* serializer,
         rv = raptor_rdfxmla_emit_blank(serializer, element, object);
         break;
 
+      case RAPTOR_IDENTIFIER_TYPE_ORDINAL:
+        /* ordinals should never appear as an object with current parsers */
+      case RAPTOR_IDENTIFIER_TYPE_PREDICATE:
+        /* predicates should never appear as an object */
+      case RAPTOR_IDENTIFIER_TYPE_UNKNOWN:
       default:
         RAPTOR_FATAL1("Unsupported identifier type\n");
         break;
@@ -530,7 +538,7 @@ raptor_rdfxmla_emit_subject_properties(raptor_serializer* serializer,
 
   i=0;
   while (!rv && i < raptor_sequence_size(subject->properties)) {
-
+    raptor_uri *base_uri=NULL;
     raptor_node *predicate;
     raptor_node *object;
     raptor_qname *qname;
@@ -553,8 +561,9 @@ raptor_rdfxmla_emit_subject_properties(raptor_serializer* serializer,
     } else
       qname = raptor_new_qname_from_resource(serializer, predicate);
     
-    element = raptor_new_xml_element(qname, NULL,
-                                     raptor_uri_copy(serializer->base_uri));
+    if(serializer->base_uri)
+      base_uri=raptor_uri_copy(serializer->base_uri);
+    element = raptor_new_xml_element(qname, NULL, base_uri);
 
     switch (object->type) {
       
@@ -574,6 +583,11 @@ raptor_rdfxmla_emit_subject_properties(raptor_serializer* serializer,
         rv = raptor_rdfxmla_emit_xml_literal(serializer, element, object);
         break;
 
+      case RAPTOR_IDENTIFIER_TYPE_ORDINAL:
+        /* ordinals should never appear as an object with current parsers */
+      case RAPTOR_IDENTIFIER_TYPE_PREDICATE:
+        /* predicates should never appear as an object */
+      case RAPTOR_IDENTIFIER_TYPE_UNKNOWN:
       default:
         RAPTOR_FATAL1("Unsupported identifier type\n");
         break;
@@ -597,6 +611,7 @@ raptor_rdfxmla_emit_subject(raptor_serializer *serializer,
   raptor_qname **attrs;
   unsigned char *attr_name;
   unsigned char *attr_value;
+  raptor_uri *base_uri=NULL;
   
   if(subject->node_type) { /* if rdf:type was associated with this subject */
     qname = raptor_new_qname_from_resource(serializer, subject->node_type);
@@ -613,8 +628,9 @@ raptor_rdfxmla_emit_subject(raptor_serializer *serializer,
                                                        (unsigned const char*)"Description",  NULL);
     
 
-  element = raptor_new_xml_element(qname, NULL,
-                                   raptor_uri_copy(serializer->base_uri));
+  if(serializer->base_uri)
+    base_uri=raptor_uri_copy(serializer->base_uri);
+  element = raptor_new_xml_element(qname, NULL, base_uri);
     
   attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 1, sizeof(raptor_qname *));
   if(!attrs)
@@ -1066,16 +1082,12 @@ raptor_subject_add_list_element(raptor_subject *subject, int ordinal,
   if(!node) {
     /* If there isn't already an entry */
     rv = raptor_sequence_set_at(subject->list_items, ordinal, object);
-
     if(!rv)
       object->ref_count++;
 
-  } else {
-    fprintf(stderr, "Object already exists in elements array at %d\n", ordinal);
   }
   
   return rv;
-  
 }
 
 
@@ -1291,7 +1303,9 @@ raptor_rdfxmla_serialize_init(raptor_serializer* serializer, const char *name)
     raptor_rdfxmla_serialize_terminate(serializer);
     return 1;
   }
-  
+
+  context->rdf_xml_literal_uri=raptor_new_uri(raptor_xml_literal_datatype_uri_string);
+
   return 0;
 }
   
@@ -1338,6 +1352,9 @@ raptor_rdfxmla_serialize_terminate(raptor_serializer* serializer)
   if(context->rdf_type)
     raptor_free_node(context->rdf_type);
   
+  if(context->rdf_xml_literal_uri)
+    raptor_free_uri(context->rdf_xml_literal_uri);
+
 }
   
 
@@ -1392,7 +1409,9 @@ raptor_rdfxmla_serialize_start(raptor_serializer* serializer)
   qname=raptor_new_qname_from_namespace_local_name(context->rdf_nspace,
                                                    (const unsigned char*)"RDF",
                                                    NULL);
-  element=raptor_new_xml_element(qname, NULL, raptor_uri_copy(base_uri));
+  if(base_uri)
+    base_uri=raptor_uri_copy(base_uri);
+  element=raptor_new_xml_element(qname, NULL, base_uri);
   context->rdf_RDF_element=element;
   for(i=0; i< raptor_sequence_size(context->namespaces); i++) {
     raptor_namespace* ns;
@@ -1420,6 +1439,7 @@ raptor_rdfxmla_serialize_statement(raptor_serializer* serializer,
   raptor_node *predicate = 0;
   raptor_node *object = 0;
   int rv;
+  raptor_identifier_type object_type;
   
   if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE ||
       statement->subject_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS ||
@@ -1435,13 +1455,21 @@ raptor_rdfxmla_serialize_statement(raptor_serializer* serializer,
     return 1;
   }  
   
-  if(statement->object_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE ||
-      statement->object_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS ||
-      statement->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL ||
-      statement->object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL || 
-      statement->object_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
+  object_type=statement->object_type;
+  if(object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL) {
+    if(statement->object_literal_datatype &&
+       raptor_uri_equals(statement->object_literal_datatype, 
+                         context->rdf_xml_literal_uri))
+      object_type = RAPTOR_IDENTIFIER_TYPE_XML_LITERAL;
+  }
 
-    object = raptor_rdfxmla_lookup_node(context, statement->object_type,
+  if(object_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE ||
+     object_type == RAPTOR_IDENTIFIER_TYPE_ANONYMOUS ||
+     object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL ||
+     object_type == RAPTOR_IDENTIFIER_TYPE_XML_LITERAL || 
+     object_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
+
+    object = raptor_rdfxmla_lookup_node(context, object_type,
                                         statement->object,
                                         statement->object_literal_datatype,
                                         statement->object_literal_language);
@@ -1449,11 +1477,12 @@ raptor_rdfxmla_serialize_statement(raptor_serializer* serializer,
       return 1;          
     
   } else {
-    raptor_serializer_error(serializer, "Do not know how to serialize node type %d\n", statement->object_type);
+    raptor_serializer_error(serializer, "Do not know how to serialize node type %d\n", object_type);
     return 1;
   }
 
-  if(statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_PREDICATE) {
+  if((statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_PREDICATE) ||
+     (statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE)) {
     predicate = raptor_rdfxmla_lookup_node(context, statement->predicate_type,
                                            statement->predicate, NULL, NULL);
 
@@ -1465,7 +1494,7 @@ raptor_rdfxmla_serialize_statement(raptor_serializer* serializer,
        * multiple type definitions.  All definitions after the
        * first go in the property list */
       subject->node_type = raptor_rdfxmla_lookup_node(context,
-                                                      statement->object_type,
+                                                      object_type,
                                                       statement->object, NULL,
                                                       NULL);
       subject->node_type->ref_count++;
@@ -1484,7 +1513,7 @@ raptor_rdfxmla_serialize_statement(raptor_serializer* serializer,
   } else if(statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
     int idx = *(int*)statement->predicate;
     rv = raptor_subject_add_list_element(subject, idx, object);
-    if(rv != 0) {
+    if(rv) {
       /* An ordinal might already exist at that location, the fallback
        * is to just put in the properties list */
       predicate = raptor_rdfxmla_lookup_node(context, statement->predicate_type,
@@ -1531,6 +1560,8 @@ raptor_rdfxmla_serialize_end(raptor_serializer* serializer)
   raptor_free_xml_element(context->rdf_RDF_element);
   context->rdf_RDF_element=NULL;
 
+  context->rdf_xml_literal_uri=raptor_new_uri(raptor_xml_literal_datatype_uri_string);
+  
   return 0;
 }
 
