@@ -1512,7 +1512,7 @@ yyreduce:
                                          prefix, 
                                          (const unsigned char*)raptor_uri_as_string(yyvsp[-1].uri), 0);
   if(yyvsp[-2].string)
-    free(yyvsp[-2].string);
+    RAPTOR_FREE(cstring, yyvsp[-2].string);
   raptor_free_uri(yyvsp[-1].uri);
 }
     break;
@@ -1682,7 +1682,7 @@ yyreduce:
 #if RAPTOR_DEBUG > 1  
   printf("resource integer=%d\n", yyvsp[0].integer);
 #endif
-  string=(unsigned char*)malloc(32); /* FIXME */
+  string=(unsigned char*)RAPTOR_MALLOC(cstring, 32); /* FIXME */
   sprintf((char*)string, "%d", yyvsp[0].integer);
   uri=raptor_new_uri((const unsigned char*)"http://www.w3.org/2001/XMLSchema#integer");
   yyval.identifier=raptor_new_identifier(RAPTOR_IDENTIFIER_TYPE_LITERAL, NULL, RAPTOR_URI_SOURCE_ELEMENT, NULL, string, uri, NULL);
@@ -2121,8 +2121,6 @@ raptor_triple_print(raptor_triple *t, FILE *fh)
 #endif
 
 
-extern char *filename;
- 
 int
 turtle_parser_error(void* ctx, const char *msg)
 {
@@ -2462,6 +2460,7 @@ main(int argc, char *argv[])
   raptor_turtle_parser turtle_parser; /* static */
   raptor_locator *locator=&rdf_parser.locator;
   FILE *fh;
+  char *filename;
 
 #if RAPTOR_DEBUG > 2
   turtle_parser_debug=1;
